@@ -5,7 +5,7 @@
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -26,7 +26,7 @@
 #include <string.h>
 
 static int fpm_event_poll_init(int max);
-static int fpm_event_poll_clean();
+static int fpm_event_poll_clean(void);
 static int fpm_event_poll_wait(struct fpm_event_queue_s *queue, unsigned long int timeout);
 static int fpm_event_poll_add(struct fpm_event_s *ev);
 static int fpm_event_poll_remove(struct fpm_event_s *ev);
@@ -50,7 +50,7 @@ static int next_free_slot = 0;
 /*
  * return the module configuration
  */
-struct fpm_event_module_s *fpm_event_poll_module() /* {{{ */
+struct fpm_event_module_s *fpm_event_poll_module(void)
 {
 #ifdef HAVE_POLL
 	return &poll_module;
@@ -58,14 +58,13 @@ struct fpm_event_module_s *fpm_event_poll_module() /* {{{ */
 	return NULL;
 #endif /* HAVE_POLL */
 }
-/* }}} */
 
 #ifdef HAVE_POLL
 
 /*
  * Init the module
  */
-static int fpm_event_poll_init(int max) /* {{{ */
+static int fpm_event_poll_init(int max)
 {
 	int i;
 
@@ -99,12 +98,11 @@ static int fpm_event_poll_init(int max) /* {{{ */
 	npollfds = max;
 	return 0;
 }
-/* }}} */
 
 /*
  * Clean the module
  */
-static int fpm_event_poll_clean() /* {{{ */
+static int fpm_event_poll_clean(void)
 {
 	/* free pollfds */
 	if (pollfds) {
@@ -121,7 +119,6 @@ static int fpm_event_poll_clean() /* {{{ */
 	npollfds = 0;
 	return 0;
 }
-/* }}} */
 
 /*
  * wait for events or timeout
@@ -136,7 +133,7 @@ static int fpm_event_poll_wait(struct fpm_event_queue_s *queue, unsigned long in
 		memcpy(active_pollfds, pollfds, sizeof(struct pollfd) * npollfds);
 	}
 
-	/* wait for inconming event or timeout */
+	/* wait for incoming event or timeout */
 	ret = poll(active_pollfds, npollfds, timeout);
 	if (ret == -1) {
 
